<?php

namespace Modules\Document\Http\Controllers;

use Illuminate\Http\Request;
use Illuminate\Http\Response;
use Illuminate\Routing\Controller;
use App\Utils\ModuleUtil;
use Menu;
use Modules\Document\Entities\Document;
use Modules\Document\Entities\DocumentShare;

class DataController extends Controller
{   
    public function superadmin_package()
    {
        return [
            [
                'name' => 'document_module',
                'label' => __('document::lang.document_module'),
                'default' => false
            ]
        ];
    }

    /**
     * Defines user permissions for the module.
     * @return array
     */
    public function user_permissions()
    {
        $permissions = [
            [
                'value' => 'access.document',
                'label' => __('document::lang.access_document'),
                'default' => false
            ],
            [
                'value' => 'create.document',
                'label' => __('document::lang.create_document'),
                'default' => false
            ]
        ];

        return $permissions;
    }
    
    /**
     * Adds Document menus
     * @return null
     */
    public function modifyAdminMenu()
    {
        $business_id = session()->get('user.business_id');
        $module_util = new ModuleUtil();
        $is_document_enabled = (boolean)$module_util->hasThePermissionInSubscription($business_id, 'document_module');

        $background_color = '';
        if (config('app.env') == 'demo') {
            $background_color = '#0086f9 !important';
        }

        if ($is_document_enabled && auth()->user()->can('access.document')) {
            Menu::modify('admin-sidebar-menu', function ($menu) use ($background_color) {
                $menu->url(
                    action('\Modules\Document\Http\Controllers\DocumentController@index'),
                    __('document::lang.document'),
                    ['icon' => 'fas fas fa-file-alt', 'active' => request()->segment(1) == 'document', 'style' => 'background-color:'.$background_color]
                        )
                ->order(90);
            });
        }
    }

    /**
     * Parses notification message from database.
     * @return array
     */
    public function parse_notification($notification)
    {   
        $notification_datas = [];
        if ($notification->type == 'Modules\Document\Notifications\DocumentShared') {
            $data = $notification->data;
            $document = Document::with('createdBy')->find($data['doc_id']);
            if (!empty($document)) {
                $msg = __(
                    'document::lang.document_shared_notif_text',
                    [
                    'shared_by' => $document->createdBy->user_full_name,
                    'name' => $document->name,
                    ]
                );
                $notification_datas = [
                    'msg' => $msg,
                    'icon_class' => 'fas fa fa-file-alt bg-blue',
                    'read_at' => $notification->read_at,
                    'link' => action('\Modules\Document\Http\Controllers\DocumentController@show', [$document->id]),
                    'created_at' => $notification->created_at->diffForHumans()
                ];
            }
        }

        return $notification_datas;
    }

    public function getSharedDocumentForGivenData($params)
    {   
        $business_id =  $params['business_id'];
        $shared_with = $params['shared_with'];
        $shared_id = $params['shared_id'];

        $sheets = DocumentShare::where('shared_with', $shared_with)
                    ->where('shared_id', $shared_id)
                    ->join('document_files as ss', 'document_file_shares.document_file_id', '=', 'ss.id')
                    ->where('ss.business_id', $business_id)
                    ->select('name as doc_name', 'document_file_id as doc_id')
                    ->get();

        return $sheets;
    }
}
