<?php

namespace Modules\InventoryReset\Http\Controllers;
use App\Product;
use App\Variation;
use App\Transaction;
use App\BusinessLocation;
use App\Utils\ModuleUtil;
use App\Utils\ProductUtil;

use Illuminate\Http\Request;
use Illuminate\Routing\Controller;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Crypt;
use Modules\InventoryReset\Entities\InventoryReset;
use Modules\InventoryReset\Entities\InventoryResetProducts;


class InventoryResetController extends Controller
{
    protected $moduleUtil;
    protected $productUtil;

    /**
     * Constructor
     *
     * @param ProductUtils $product
     * @return void
     */
    public function __construct(ProductUtil $productUtil, ModuleUtil $moduleUtil)
    {
        $this->productUtil = $productUtil;
        $this->moduleUtil = $moduleUtil;
    }
    public function index()
    {
        $business_id = request()->session()->get('user.business_id');
        $locations = BusinessLocation::where('business_id', $business_id)->select(['business_locations.name','business_locations.id'])->get();
        $resets = InventoryReset::where('business_id', $business_id)->with('createdBy')->latest()->get();
        
        return view('inventoryreset::index',compact('resets','locations'));
    }
    public function show($id)
    {
        
        $business_id = request()->session()->get('user.business_id');

        $resets = InventoryReset::where('id',$id)
            ->where('business_id', $business_id)
            ->with('product','product.branch','product.variation.product')
            ->firstOrFail();
        $resetProducts = $resets->product;

        $groupbedContent = $resetProducts->groupBy('branch.name');
        // $branches = $resetProducts->pluck('branch.name','branch.id');
        // dd($branches);
        return view('inventoryreset::show',compact('groupbedContent'));
    }
            public function store(Request $request)
            {
                 // Check if app environment is set to demo
    if (config('app.env') === 'demo') {
        $output = [
            'success' => 0,
            'msg' => 'Feature disabled in demo!!',
        ];

        return back()->with('status', $output);
    }

              
                $locations = [];
                foreach ($request->location_id as $location) {
                    try {
                        $locations[] = Crypt::decryptString($location);
                    } catch (\Exception $e) {
                        \Log::emergency("File:" . $e->getFile() . "Line:" . $e->getLine() . "Message:" . $e->getMessage());
                        $output = ['success' => 0, 'msg' => __("messages.something_went_wrong")];
                        return abort(400, $output);
                    }
                }
               
                if (!auth()->user()->can('inventoryreset.reset')) {
                    abort(403, 'Unauthorized action.');
                }
                if (!(auth()->user()->can('inventoryreset') || ($this->moduleUtil->hasThePermissionInSubscription($business_id, 'inventoryreset_module')))) {
                    abort(403, 'Unauthorized action.');
                }
               
                $business_id = request()->session()->get('user.business_id');
                $location = BusinessLocation::where('business_id', $business_id)->whereIn('id', $locations)->get();
                $locations = $location->pluck('id');
                $user_id = $request->session()->get('user.id');
            
                try {
                    DB::beginTransaction();
                    $invReset = InventoryReset::create(['business_id' => $business_id, 'created_by' => request()->session()->get('user.id')]);
                    foreach ($locations as $location_id) {
                        $products = Product::where('business_id', $business_id)->with('variations')->get();
                        if ($products->count() > 0) {
                            $variations = Variation::whereIn('product_id', $products->pluck('id'))
                                ->whereHas('variation_location_details', function ($q) use ($location_id) {
                                    $q->where('location_id', $location_id);
                                })->with(['variation_location_details' => function ($q) use ($location_id) {
                                    $q->where('location_id', $location_id);
                                }])
                                ->get();
                            if ($variations->count() > 0) {
                                foreach ($variations as $var) {
                                    $transaction = Transaction::create([
                                        'business_id' => $business_id,
                                        'created_by' => $user_id,
                                        'transaction_date' => now(),
                                        'status' => 'final',
                                        'type' => 'reset',
                                        'location_id' => $location_id,
                                    ]);
                                    $product_qty = $var->variation_location_details->isEmpty() ? 0 : $var->variation_location_details[0]->qty_available;
                                    InventoryResetProducts::create([
                                        'variation_id' => $var->id,
                                        'branch_id' => $location_id,
                                        'transaction_id' => $transaction->id,
                                        'inventories_reset_id' => $invReset->id,
                                        'qty_before' => $product_qty,
                                    ]);
                                    if ($product_qty != 0) {
                                        $updateQty = $this->productUtil
                                            ->updateProductQuantity($location_id, $var->product_id, $var->id, -abs($product_qty));
                                    }
                                    // Set product quantity to 0
                                    $this->productUtil->updateProductQuantity($location_id, $var->product_id, $var->id, 0);
                                }
                            }
                        }
                    }
                    DB::commit();
                    return response()->json(['status' => true, 'msg' => 'Inventory reset successful.']);
                } catch (\Exception $e) {
                   
            
        DB::rollBack();
        \Log::emergency("File:" . $e->getFile() . "Line:" . $e->getLine() . "Message:" . $e->getMessage());
            return [
                    'success'   => 0,
                    'msg'       => __("messages.something_went_wrong")
                ];
        }



    }
}

